import 'package:augnito_flutter_sdk/config/augnito_config.dart';
import 'package:augnito_flutter_sdk/support/domain_utils.dart';

class SDKConfig {
  static const int interval = 100;

  late final String _domainUrl;
  late final String _macroServiceURL;
  late final String _pushNotificationUrl;
  late final String _speechMicUrl;
  final AugnitoConfig _augnitoConfig;
  late final String _speechUrl;

  // getters
  AugnitoConfig get clientConfig {
    return _augnitoConfig;
  }

  String get macroServiceUrl {
    return _macroServiceURL;
  }

  String get pushNotificationUrl {
    return _pushNotificationUrl;
  }

  String get speechMicUrl {
    return _speechMicUrl;
  }

  String get speechUrl => _speechUrl;

  SDKConfig(this._augnitoConfig) {
    final domainName = DomainUtils.getDomainName(_augnitoConfig.server);
    assert(domainName.isNotEmpty,
        "Unable to determine domain name. Verify you are providing a valid server.");
    _domainUrl = "wss://$domainName/v2/speechapi";
    _macroServiceURL = "https://$domainName/manage/v2";
    _pushNotificationUrl = "wss://$domainName/speechapi/notification/";
    _speechMicUrl = "wss://$domainName/speechapi/mobile/client/";
    _speechUrl = _buildSpeechUrl();
  }

  String _buildSpeechUrl() {
    var stringBuffer = StringBuffer();
    stringBuffer.write(_domainUrl);
    stringBuffer.write("?content-type=${_augnitoConfig.contentType}");
    stringBuffer.write("&accountcode=${_augnitoConfig.accountCode}");
    stringBuffer.write("&accesskey=${_augnitoConfig.accessKey}");
    stringBuffer.write("&lmid=${_augnitoConfig.lmid}");
    stringBuffer.write("&usertag=${_augnitoConfig.userTag}");
    stringBuffer.write("&logintoken=${_augnitoConfig.loginToken ?? ""}");
    stringBuffer.write("&noisect=${_augnitoConfig.noiseCt}");
    stringBuffer.write("&otherinfo=${_augnitoConfig.otherInfo ?? ""}");
    stringBuffer.write("&sourceapp=${_augnitoConfig.sourceApp}");

    return stringBuffer.toString();
  }
}
