import 'dart:collection';

import '../../models/action_recipe.dart';
import 'processors/active_x_command_processor.dart';
import 'processors/generic_goto_processor.dart';
import 'processors/goto_moveto_command_processor.dart';
import 'processors/goto_moveto_paragraph_command_processor.dart';
import 'processors/goto_x_command_processor.dart';
import 'processors/line_paragraph_number_command_processor.dart';
import 'processors/navigation_without_goto_processor.dart';
import 'processors/number_command_processor.dart';
import 'processors/base_regex_command_processor.dart';
import 'processors/replace_command_processor.dart';
import 'processors/select_group_processor.dart';
import 'processors/select_item_direction_processor.dart';

abstract class AugnitoDynamicCommandProcessor {
  static final RegExp SET_FONT_SIZE = RegExp(
      r"^setfontsize(to)?([0-9]+)(point[s]?)?$",
      caseSensitive: false,
      multiLine: false);

  static final RegExp GO_TO_FIELDS = RegExp(
      r"^(select|go|goto|gotothe|move|moveto|movethe)?(field[s]?|filled)(number[ed]?)?([0-9]+)$",
      caseSensitive: false,
      multiLine: false);

  static final RegExp REVOVE_SPACE_BEFORE_PUNCTUATION =
      RegExp(r"^ ([.|:|,])(.*?)$", caseSensitive: false, multiLine: false);

  static final UnmodifiableListView<BaseRegexCommandProcessor> regexProcessors =
      _populateAndPrepareRegexProcessors();

  /// Initializes the list of processors and calls the sort method to ensure
  /// they are called in the correct order
  static UnmodifiableListView<BaseRegexCommandProcessor>
      _populateAndPrepareRegexProcessors() {
    final result = [
      LineParagraphNumberCommandProcessor(),
      GoToMoveToCommandProcessor(),
      GoToMoveToParagraphCommandProcessor(),
      ActiveXCommandProcessor(),
      NavigationWithoutGoToProcessor(),
      GotoXCommandProcessor(),
      SelectItemDirectionProcessor(),
      ReplaceXWithYCommandProcessor(),
      SelectGroupProcessor(),
      NumberCommandProcessor(),
      GenericGoToCommandProcessor()
    ];
    result.sort((a, b) => a.precedence.compareTo(b.precedence));
    return UnmodifiableListView(result);
  }

  /// Given a recipe and a command attempts to determine the final command
  /// modifying the [ActionRecipe] received
  static void fillDynamicCommand(ActionRecipe recipe, String cmdText) {
    // attempt regex processors
    for (var processor in regexProcessors) {
      if (processor.processActionRecipe(recipe, cmdText)) {
        return;
      }
    }
  }
}
