import '../../../models/action_recipe.dart';
import '../../../support/augnito_commands.dart';
import '../../utils/command_utils.dart';
import 'base_regex_command_processor.dart';

/// Attempts to process the Active X command.
/// ie: select current space
/// returns false if unable to fullfil all conditions
class ActiveXCommandProcessor implements BaseRegexCommandProcessor {
  static final RegExp _activeX = RegExp(
      r"^(" +
          BaseRegexCommandProcessor.actionCMD +
          r")(the)?(active|current)?(word[s]?|line[s]?|sentence[s]?|paragraph[s]?|para[s]?|char[s]?|character[s]?|space|\n\n|\n\ns|\n|\ns)$",
      caseSensitive: false,
      multiLine: false);

  static ActiveXCommandProcessor? _instance;
  ActiveXCommandProcessor._internal();

  factory ActiveXCommandProcessor() {
    return _instance ?? ActiveXCommandProcessor._internal();
  }

  @override
  bool processActionRecipe(ActionRecipe recipe, String regexTestString) {
    final activeX = _activeX.firstMatch(regexTestString);

    if (activeX != null && activeX.groupCount > 3) {
      final cmdAction =
          CommandUtils.setActionCommandVariant(activeX.group(1) ?? "");
      final item = (activeX.group(4) ?? "").toLowerCase();
      if (item.isNotEmpty && cmdAction.isNotEmpty) {
        recipe.name = CommandUtils.setActiveObjectType(item);
        recipe.receivedText = "";
        recipe.searchText = item;
        recipe.isCommand = true;
        recipe.selectFor = cmdAction == AugnitoCommands.select ? "" : cmdAction;
        return true;
      }
    }
    return false;
  }

  @override
  int get precedence => 3;
}
