import '../../../models/action_recipe.dart';
import '../../utils/word_to_integer_utils.dart';
import '../../utils/command_utils.dart';
import 'base_regex_command_processor.dart';

/// Attempts to process the Go To X command.
/// ie: goto last 3 sentences
/// returns false if unable to fullfil all conditions
class GotoXCommandProcessor implements BaseRegexCommandProcessor {
  static final RegExp _goToX = RegExp(
      r"^(go|goto|gotothe|move|moveto|movethe)(last|previous|next|down)(.*?)(word[s]?|line[s]?|sentence[s]?|paragraph[s]?|para[s]?|char[s]?|character[s]?|space|\n\n|\n)$",
      caseSensitive: false,
      multiLine: false);

  static GotoXCommandProcessor? _instance;
  GotoXCommandProcessor._internal();

  factory GotoXCommandProcessor() {
    return _instance ?? GotoXCommandProcessor._internal();
  }

  @override
  bool processActionRecipe(ActionRecipe recipe, String regexTestString) {
    final selectStartOfX = _goToX.firstMatch(regexTestString.trim());
    if (selectStartOfX != null && selectStartOfX.groupCount > 2) {
      final direction = selectStartOfX.group(2) ?? "";
      final wordCount = WordToIntegerUtils.parse(selectStartOfX.group(3) ?? "");
      final item = (selectStartOfX.group(4) ?? "").toLowerCase();
      recipe.name = CommandUtils.setObjectType(item);
      recipe.nextPrevious = direction;
      recipe.chooseNumber = wordCount;
      recipe.receivedText = "";
      recipe.isCommand = true;
      recipe.selectFor = (direction == "previous" || direction == "last")
          ? "gotostart"
          : "gotoend";
      return true;
    }
    return false;
  }

  @override
  int get precedence => 5;
}
