import '../../../models/action_recipe.dart';
import '../../../support/augnito_commands.dart';
import '../../utils/word_to_integer_utils.dart';
import '../../utils/command_utils.dart';
import 'base_regex_command_processor.dart';

/// Attempts to process a Line Paragraph Number.
/// ie: select line number 10
/// returns false if unable to fullfil all conditions
class LineParagraphNumberCommandProcessor implements BaseRegexCommandProcessor {
  static final RegExp _lineParagraphNumber = RegExp(
      r"^(" +
          BaseRegexCommandProcessor.actionCMD +
          r")(the)?(line|para|paragraph|\n\n|\n)(number[ed]?)(.*?)(\.)?$",
      caseSensitive: false,
      multiLine: false);

  static LineParagraphNumberCommandProcessor? _instance;
  LineParagraphNumberCommandProcessor._internal();

  factory LineParagraphNumberCommandProcessor() {
    return _instance ?? LineParagraphNumberCommandProcessor._internal();
  }

  @override
  bool processActionRecipe(ActionRecipe recipe, String regexTestString) {
    final lineSelect = _lineParagraphNumber.firstMatch(regexTestString.trim());

    if (lineSelect != null && lineSelect.groupCount > 2) {
      var cmdAction = lineSelect.group(1) ?? "";
      var lineOrPara = lineSelect.group(3) ?? "";
      if (cmdAction.isNotEmpty) {
        cmdAction =
            CommandUtils.setActionCommandVariant(cmdAction.toLowerCase());
        final regexLineNumber = lineSelect.group(5);
        int lineNumber = WordToIntegerUtils.parse(regexLineNumber ?? "");
        if (lineNumber > 0 && cmdAction.isNotEmpty) {
          recipe.name = lineOrPara == "line" || lineOrPara == "\n"
              ? AugnitoCommands.goToLineNumber
              : AugnitoCommands.goToParaNumber;
          recipe.chooseNumber = lineNumber;
          recipe.searchText = lineOrPara;
          recipe.isCommand = true;
          recipe.selectFor =
              cmdAction == AugnitoCommands.select ? "" : cmdAction;
          return true;
        }
      }
    }
    return false;
  }

  @override
  int get precedence => 0;
}
