import '../../../models/action_recipe.dart';
import '../../utils/command_utils.dart';
import 'base_regex_command_processor.dart';

/// Attempts to process a navigation without "Go To".
/// ie: last 3 space
/// ie: last 3 words
/// returns false if unable to fullfil all conditions
class NavigationWithoutGoToProcessor implements BaseRegexCommandProcessor {
  static final RegExp _navigationWithoutGoTo = RegExp(
      r"^(last|previous|next|down)(.*?)(word[s]?|line[s]?|sentence[s]?|paragraph[s]?|para[s]?|char[s]?|character[s]?|space|\n\n|\n)$",
      caseSensitive: false,
      multiLine: false);

  static NavigationWithoutGoToProcessor? _instance;
  NavigationWithoutGoToProcessor._internal();

  factory NavigationWithoutGoToProcessor() {
    return _instance ?? NavigationWithoutGoToProcessor._internal();
  }

  @override
  bool processActionRecipe(ActionRecipe recipe, String regexTestString) {
    final selectNavigationWIthoutGoto =
        _navigationWithoutGoTo.firstMatch(regexTestString.trim());

    if (selectNavigationWIthoutGoto != null &&
        selectNavigationWIthoutGoto.groupCount > 2) {
      final direction =
          (selectNavigationWIthoutGoto.group(1) ?? "").toLowerCase();
      final item = (selectNavigationWIthoutGoto.group(3) ?? "").toLowerCase();
      recipe.name = CommandUtils.setObjectType(item);
      recipe.nextPrevious = direction;
      recipe.chooseNumber = 1; // TODO: should this match the number ?
      recipe.receivedText = "";
      recipe.isCommand = true;
      recipe.selectFor = (direction == "previous" || direction == "last")
          ? "gotostart"
          : "gotoend";
      return true;
    }
    return false;
  }

  @override
  int get precedence => 4;
}
