import '../../../models/action_recipe.dart';
import '../../../support/augnito_commands.dart';
import 'base_regex_command_processor.dart';

/// Attempts to process number selection command.
/// ie: number 10
/// ie: 12
/// returns false if unable to fullfil all conditions
class NumberCommandProcessor implements BaseRegexCommandProcessor {
  static final RegExp _numberChoose = RegExp(r"^(number)?([0-9]+|x)[.]?$",
      caseSensitive: false, multiLine: false);

  static NumberCommandProcessor? _instance;
  NumberCommandProcessor._internal();

  factory NumberCommandProcessor() {
    return _instance ?? NumberCommandProcessor._internal();
  }

  @override
  bool processActionRecipe(ActionRecipe recipe, String regexTestString) {
    if (recipe.receivedText.length < 12) {
      final numberChoose = _numberChoose.firstMatch(regexTestString);
      if (numberChoose != null && numberChoose.groupCount > 1) {
        var dig = numberChoose.group(2) ?? "";
        dig = dig == "x" ? "10" : dig;
        final parsed = int.tryParse(dig);
        if (parsed != null) {
          recipe.name = AugnitoCommands.number;
          recipe.chooseNumber = parsed;
          recipe.isCommand =
              false; // Will consider this has command only when system is some selection mode otherwise type.
          return true;
        }
      }
    }
    return false;
  }

  @override
  int get precedence => 9;
}
