import '../../../models/action_recipe.dart';
import '../../../support/augnito_commands.dart';
import 'base_regex_command_processor.dart';

/// Attempts to process the replace X with Y command.
/// ie: replace cat with dog
/// returns false if unable to fullfil all conditions
class ReplaceXWithYCommandProcessor implements BaseRegexCommandProcessor {
  static final RegExp _replaceXWithY = RegExp(
      r"replace[d]?([A-Z a-z 0-9]+)with([A-Z a-z 0-9]+)",
      caseSensitive: false,
      multiLine: false);

  static ReplaceXWithYCommandProcessor? _instance;
  ReplaceXWithYCommandProcessor._internal();

  factory ReplaceXWithYCommandProcessor() {
    return _instance ?? ReplaceXWithYCommandProcessor._internal();
  }

  @override
  bool processActionRecipe(ActionRecipe recipe, String regexTestString) {
    final replace = _replaceXWithY.firstMatch(regexTestString.trim());
    if (replace != null && replace.groupCount == 2) {
      recipe.name = AugnitoCommands.replace;
      recipe.searchText = (replace.group(1) ?? "").trim();
      recipe.selectFor = (replace.group(2) ?? "").trim();
      recipe.isCommand = true;
      return true;
    }
    return false;
  }

  @override
  int get precedence => 7;
}
