import '../../../models/action_recipe.dart';
import '../../../support/augnito_commands.dart';
import '../../utils/command_utils.dart';
import 'base_regex_command_processor.dart';

/// Attempts to process the select command.
/// ie: select patient
/// returns false if unable to fullfil all conditions
class SelectGroupProcessor implements BaseRegexCommandProcessor {
  static final RegExp _selectGroup2 = RegExp(
      r"^(" + BaseRegexCommandProcessor.actionCMD + r")(\sthe)?\s?(.*?)$",
      caseSensitive: false,
      multiLine: false);

  static SelectGroupProcessor? _instance;
  SelectGroupProcessor._internal();

  factory SelectGroupProcessor() {
    return _instance ?? SelectGroupProcessor._internal();
  }

  @override
  bool processActionRecipe(ActionRecipe recipe, String regexTestString) {
    final selectOps = _selectGroup2.firstMatch(recipe.receivedText.trim());
    if (selectOps != null && selectOps.groupCount > 2) {
      final cmdAction =
          CommandUtils.setActionCommandVariant(selectOps.group(1) ?? "");
      final searchText = selectOps.group(3) ?? "";
      if (searchText.isNotEmpty) {
        recipe.name = AugnitoCommands.select;
        recipe.searchText = searchText.trim();
        recipe.isCommand = true;
        recipe.selectFor = cmdAction == AugnitoCommands.select ? "" : cmdAction;
        return true;
      }
    }
    return false;
  }

  @override
  int get precedence => 8;
}
