import '../../../models/action_recipe.dart';
import '../../../support/augnito_commands.dart';
import '../../utils/command_utils.dart';
import '../../utils/word_to_integer_utils.dart';
import 'base_regex_command_processor.dart';

/// Attempts to process the select/action word or line + direction.
/// ie: delete the previous 3 words
/// returns false if unable to fullfil all conditions
class SelectItemDirectionProcessor implements BaseRegexCommandProcessor {
  static final RegExp _selectWordLineNextPrevious = RegExp(
      r"^(" +
          BaseRegexCommandProcessor.actionCMD +
          r")(the)?(last|previous|next)(.*?)(word[s]?|line[s]?|sentence[s]?|paragraph[s]?|para[s]?|char[s]?|character[s]?|space|\n\n|\n\ns|\n|\ns)$",
      caseSensitive: false,
      multiLine: false);

  static SelectItemDirectionProcessor? _instance;
  SelectItemDirectionProcessor._internal();

  factory SelectItemDirectionProcessor() {
    return _instance ?? SelectItemDirectionProcessor._internal();
  }

  @override
  bool processActionRecipe(ActionRecipe recipe, String regexTestString) {
    final selectManyWord =
        _selectWordLineNextPrevious.firstMatch(regexTestString);
    if (selectManyWord != null && selectManyWord.groupCount > 2) {
      final cmdAction =
          CommandUtils.setActionCommandVariant(selectManyWord.group(1) ?? "");
      final direction = selectManyWord.group(3) ?? "";
      var wordCount = WordToIntegerUtils.parse(selectManyWord.group(4) ?? "");
      wordCount = wordCount == 0 ? 1 : wordCount;
      final item = (selectManyWord.group(5) ?? "").toLowerCase();
      if (item.isNotEmpty && cmdAction.isNotEmpty && direction.isNotEmpty) {
        recipe.name = CommandUtils.setObjectType(item);
        recipe.nextPrevious = direction;
        recipe.chooseNumber = wordCount;
        recipe.receivedText = "";
        recipe.searchText = item;
        recipe.isCommand = true;
        recipe.selectFor = cmdAction == AugnitoCommands.select ? "" : cmdAction;
        return true;
      }
    }
    return false;
  }

  @override
  int get precedence => 6;
}
