import '../../models/action_recipe.dart';
import '../../support/augnito_commands.dart';

abstract class AugnitoStaticCommandProcessor {
  /// Transforms an [ActionRecipe] to it's final form when it's been received
  /// as a Static Command.
  static formatStaticCommand(ActionRecipe recipe) {
    switch (recipe.action) {
      case AugnitoCommands.selectAll:
        recipe.name = AugnitoCommands.select;
        recipe.searchText = "all";
        return;
      case AugnitoCommands.selectPreviousWord:
        recipe.name = AugnitoCommands.selectWord;
        recipe.chooseNumber = 1;
        recipe.nextPrevious = "last";
        recipe.searchText = "previousword";
        return;
      case AugnitoCommands.selectNextWord:
        recipe.name = AugnitoCommands.selectWord;
        recipe.chooseNumber = 1;
        recipe.nextPrevious = "next";
        recipe.searchText = "nextword";
        return;
      case AugnitoCommands.deleteLastWord:
        recipe.name = AugnitoCommands.selectWord;
        recipe.chooseNumber = 1;
        recipe.nextPrevious = "previous";
        recipe.selectFor = "delete";
        return;
      case AugnitoCommands.deleteLastLine:
        recipe.name = AugnitoCommands.selectLine;
        recipe.chooseNumber = 1;
        recipe.nextPrevious = "previous";
        recipe.selectFor = "delete";
        return;
      case AugnitoCommands.boldLastWord:
        recipe.name = AugnitoCommands.selectWord;
        recipe.chooseNumber = 1;
        recipe.nextPrevious = "previous";
        recipe.selectFor = "bold";
        return;
      case AugnitoCommands.boldLastLine:
        recipe.name = AugnitoCommands.selectLine;
        recipe.chooseNumber = 1;
        recipe.nextPrevious = "previous";
        recipe.selectFor = "bold";
        return;
      case AugnitoCommands.selectPreviousParagraph:
        recipe.name = AugnitoCommands.selectParagraph;
        recipe.chooseNumber = 1;
        recipe.nextPrevious = "previous";
        return;
      case AugnitoCommands.selectNextLine:
        recipe.name = AugnitoCommands.selectLine;
        recipe.chooseNumber = 1;
        recipe.nextPrevious = "next";
        return;
      case AugnitoCommands.selectPreviousLine:
        recipe.name = AugnitoCommands.selectLine;
        recipe.chooseNumber = 1;
        recipe.nextPrevious = "previous";
        return;
      case AugnitoCommands.selectNextParagraph:
        recipe.name = AugnitoCommands.selectParagraph;
        recipe.chooseNumber = 1;
        recipe.nextPrevious = "next";
        return;
      case AugnitoCommands.backspace:
        recipe.name = AugnitoCommands.selectChar;
        recipe.chooseNumber = 1;
        recipe.nextPrevious = "previous";
        recipe.selectFor = "delete";
        return;
      default:
        return;
    }
  }
}
