import '../../models/action_recipe.dart';
import '../../support/augnito_commands.dart';

abstract class CommandUtils {
  /// item dictionary lookup
  static const Map<String, String> _itemDictionary = {
    'word': AugnitoCommands.selectWord,
    'words': AugnitoCommands.selectWord,
    'line': AugnitoCommands.selectLine,
    'lines': AugnitoCommands.selectLine,
    'paragraph': AugnitoCommands.selectParagraph,
    'paragraphs': AugnitoCommands.selectParagraph,
    'para': AugnitoCommands.selectParagraph,
    'paras': AugnitoCommands.selectParagraph,
    '\n\n': AugnitoCommands.selectParagraph,
    '\n\ns': AugnitoCommands.selectParagraph,
    '\n': AugnitoCommands.selectLine,
    '\ns': AugnitoCommands.selectLine,
    'char': AugnitoCommands.selectChar,
    'chars': AugnitoCommands.selectChar,
    'space': AugnitoCommands.selectChar,
    'character': AugnitoCommands.selectChar,
    'characters': AugnitoCommands.selectChar,
  };

  /// active item dictionary lookup
  static const Map<String, String> _activeItemDictionary = {
    'word': AugnitoCommands.selectActiveWord,
    'words': AugnitoCommands.selectActiveWord,
    'line': AugnitoCommands.selectActiveLine,
    'lines': AugnitoCommands.selectActiveLine,
    'paragraph': AugnitoCommands.selectActiveParagraph,
    'paragraphs': AugnitoCommands.selectActiveParagraph,
    'para': AugnitoCommands.selectActiveParagraph,
    'paras': AugnitoCommands.selectActiveParagraph,
    '\n\n': AugnitoCommands.selectActiveParagraph,
    '\n\ns': AugnitoCommands.selectActiveParagraph,
    '\n': AugnitoCommands.selectLine,
    '\ns': AugnitoCommands.selectLine,
    'char': AugnitoCommands.selectActiveChar,
    'chars': AugnitoCommands.selectActiveChar,
    'space': AugnitoCommands.selectActiveChar,
    'character': AugnitoCommands.selectActiveChar,
    'characters': AugnitoCommands.selectActiveChar,
  };

  /// action variant lookup
  static const Map<String, String> _actionCommandVariants = {
    'd capitalise': 'uncapitalize',
    'd capitalize': 'uncapitalize',
    'decapitalize': 'uncapitalize',
    'dcapitalise': 'uncapitalize',
    'decapitalicize': 'uncapitalize',
    'dcapitalize': 'uncapitalize',
    'uncapitalize': 'uncapitalize',
    'uncap': 'uncapitalize',
    'd underline': 'deunderline',
    'dunderline': 'deunderline',
    'ununderline': 'deunderline',
    'capitalise': 'capitalize',
    'remove': 'delete',
    'goto': 'gotostart',
    'move': 'gotostart',
    'movto': 'gotostart',
    'debold': 'unbold',
    'dbold': 'unbold',
    'cuttext': 'cut',
    'cut text': 'cut',
    'copytext': 'copy',
    'copy text': 'copy',
    'choose': 'select',
    'italicise': 'italicize',
    'unitalicise': 'unitalicize',
  };

  /// attempts to retrieve the object type from the dictionary.Returns the input
  /// if no entry was found.
  /// ie: 'word' produces CMDString.SELECT_WORD
  static String setObjectType(String item) {
    return _itemDictionary[item] ?? item;
  }

  /// attempts to retrieve the object type from the dictionary.Returns the input
  /// if no entry was found.
  /// ie: 'word' produces  SELECT_ACTIVE_WORD
  static String setActiveObjectType(String item) {
    return _activeItemDictionary[item] ?? item;
  }

  /// attempts to find an action variant. If no variant is found the input is returned.
  /// ie: cut text produces cut
  /// ie: d capitalise produces uncapitalize
  static String setActionCommandVariant(String action) {
    return _actionCommandVariants[action] ?? action;
  }

  static final RegExp _newLineAroundSpace =
      RegExp(r"[^\S\n]*(\n+)[^\S\n]*", caseSensitive: false, multiLine: false);

  /// Trims spaces between and after a new line
  static void triNewLine(ActionRecipe recipe) {
    if (_newLineAroundSpace.hasMatch(recipe.receivedText)) {
      for (final match in _newLineAroundSpace.allMatches(recipe.receivedText)) {
        final subString = match.group(0) ?? "";
        final replace = match.group(1) ?? "";
        if (subString.isNotEmpty && replace.isNotEmpty) {
          recipe.receivedText =
              recipe.receivedText.replaceAll(subString, replace);
        }
      }
    }
  }
}
