import 'dart:io';

import '../../support/logging/augnito_print_logger.dart';

/// Handles communication with Augnito's server using WebSocket from dart:io
class AugnitoWebClient {
  WebSocket? _webSocket;
  final String _speechURL;

  Function(dynamic)? _onMessage;
  Function(dynamic)? _onError;
  Future<void> Function()? _onDisconnected;

  AugnitoPrintLogger? _logger;

  AugnitoWebClient(this._speechURL,
      {Function(dynamic)? onMessage,
      Function(dynamic)? onError,
      Future<void> Function()? onDisconnected,
      enableLogs = false}) {
    _onMessage = onMessage;
    _onError = onError;
    _onDisconnected = onDisconnected;
    _logger = enableLogs ? AugnitoPrintLogger() : null;
  }

  /// Attempts to open the connection with the server. Returns true if
  /// successful and false otherwise.
  Future<bool> open() async {
    try {
      _webSocket = await WebSocket.connect(_speechURL);
    } on SocketException catch (e) {
      _logger?.log(this, e);
      return false;
    } on Exception catch (e) {
      _logger?.log(this, e);
      return false;
    }

    _webSocket?.pingInterval = const Duration(seconds: 5);
    _webSocket?.listen(
        (data) {
          if (_onMessage != null) {
            _onMessage!(data);
          }
        },
        cancelOnError: true,
        onDone: () {
          if (_onDisconnected != null) {
            _onDisconnected!();
          }
          _webSocket = null;
        },
        onError: (error) {
          if (_onError != null) {
            _onError!(error);
          }
        });
    return true;
  }

  /// Sends a message of type List<int> through the socket. Intended to be
  /// used with audio data.
  void sendMessage(List<int> data) {
    _webSocket?.add(data);
  }

  /// Closes the underlying websocket.
  void close() {
    _webSocket?.close(WebSocketStatus.goingAway);
  }
}
