import 'package:augnito_flutter_sdk/dictation/models/action_recipe.dart';
import 'package:augnito_flutter_sdk/dictation/support/augnito_commands.dart';
import 'package:augnito_flutter_sdk/dictation/commands/augnito_command_builder.dart';
import 'package:augnito_flutter_sdk/text/utils/text_field_processor.dart';
import 'package:flutter_test/flutter_test.dart';

void main() {
  TestWidgetsFlutterBinding.ensureInitialized();

  setUp(() {});

  tearDown(() {});

  test("lineParagraphNumber: select line number 10", () {
    var augnitoRecipe = ActionRecipe(receivedText: "select line number 10");
    AugnitoCommandBuilder.build(augnitoRecipe);
    expect(augnitoRecipe.name, AugnitoCommands.goToLineNumber);
    expect(augnitoRecipe.isCommand, true);
    expect(augnitoRecipe.chooseNumber, 10);
    expect(augnitoRecipe.selectFor, "");
  });

  test("lineParagraphNumber: bold paragraph number 2", () {
    var augnitoRecipe = ActionRecipe(receivedText: "bold paragraph number 2");
    AugnitoCommandBuilder.build(augnitoRecipe);
    expect(augnitoRecipe.name, AugnitoCommands.goToParaNumber);
    expect(augnitoRecipe.isCommand, true);
    expect(augnitoRecipe.chooseNumber, 2);
    expect(augnitoRecipe.selectFor, "bold");
  });

  test("lineParagraphNumber: Uncap the line number 5", () {
    var augnitoRecipe = ActionRecipe(receivedText: "Uncap the line number 5");
    AugnitoCommandBuilder.build(augnitoRecipe);
    expect(augnitoRecipe.name, AugnitoCommands.goToLineNumber);
    expect(augnitoRecipe.isCommand, true);
    expect(augnitoRecipe.chooseNumber, 5);
    expect(augnitoRecipe.selectFor, "uncapitalize");
  });

  test("goto new line", () {
    var augnitoRecipe = ActionRecipe(receivedText: "goto \n");
    AugnitoCommandBuilder.build(augnitoRecipe);
    expect(augnitoRecipe.name, AugnitoCommands.selectLine);
    expect(augnitoRecipe.isCommand, true);
    expect(augnitoRecipe.chooseNumber, 1);
    expect(augnitoRecipe.receivedText, "");
    expect(augnitoRecipe.selectFor, "gotoend");
  });

  test("move to new para", () {
    var augnitoRecipe = ActionRecipe(receivedText: "moveto \n\n");
    AugnitoCommandBuilder.build(augnitoRecipe);
    expect(augnitoRecipe.name, AugnitoCommands.selectParagraph);
    expect(augnitoRecipe.isCommand, true);
    expect(augnitoRecipe.chooseNumber, 1);
    expect(augnitoRecipe.receivedText, "");
    expect(augnitoRecipe.selectFor, "gotoend");
  });

  /// TODO: ask why there are no setnence related ???? inconsistency
  test("select active sentence", () {
    var augnitoRecipe = ActionRecipe(receivedText: "select active sentence");
    AugnitoCommandBuilder.build(augnitoRecipe);
    expect(augnitoRecipe.name, "sentence");
    expect(augnitoRecipe.isCommand, true);
    expect(augnitoRecipe.searchText, "sentence");
    expect(augnitoRecipe.receivedText, "");
    expect(augnitoRecipe.selectFor, "");
  });

  test("select the active paragraph", () {
    var augnitoRecipe =
        ActionRecipe(receivedText: "select the active paragraph");
    AugnitoCommandBuilder.build(augnitoRecipe);
    expect(augnitoRecipe.name, AugnitoCommands.selectActiveParagraph);
    expect(augnitoRecipe.isCommand, true);
    expect(augnitoRecipe.searchText, "paragraph");
    expect(augnitoRecipe.receivedText, "");
    expect(augnitoRecipe.selectFor, "");
  });

  test("select the active word", () {
    var augnitoRecipe = ActionRecipe(receivedText: "select the active word");
    AugnitoCommandBuilder.build(augnitoRecipe);
    expect(augnitoRecipe.name, AugnitoCommands.selectActiveWord);
    expect(augnitoRecipe.isCommand, true);
    expect(augnitoRecipe.searchText, "word");
    expect(augnitoRecipe.receivedText, "");
    expect(augnitoRecipe.selectFor, "");
  });

  test("underline active word", () {
    var augnitoRecipe = ActionRecipe(receivedText: "underline active word");
    AugnitoCommandBuilder.build(augnitoRecipe);
    expect(augnitoRecipe.name, AugnitoCommands.selectActiveWord);
    expect(augnitoRecipe.isCommand, true);
    expect(augnitoRecipe.searchText, "word");
    expect(augnitoRecipe.receivedText, "");
    expect(augnitoRecipe.selectFor, "underline");
  });

  /// TODO: shouldn't choose number be reflecting the unit ?
  test("next 3 words", () {
    var augnitoRecipe = ActionRecipe(receivedText: "next 3 words");
    AugnitoCommandBuilder.build(augnitoRecipe);
    expect(augnitoRecipe.name, AugnitoCommands.selectWord);
    expect(augnitoRecipe.isCommand, true);
    expect(augnitoRecipe.chooseNumber, 1);
    expect(augnitoRecipe.receivedText, "");
    expect(augnitoRecipe.nextPrevious, "next");
    expect(augnitoRecipe.selectFor, "gotoend");
  });

  test("goto last 3 sentences", () {
    var augnitoRecipe = ActionRecipe(receivedText: "goto last 3 sentences");
    AugnitoCommandBuilder.build(augnitoRecipe);
    expect(augnitoRecipe.name, "sentences");
    expect(augnitoRecipe.isCommand, true);
    expect(augnitoRecipe.chooseNumber, 3);
    expect(augnitoRecipe.receivedText, "");
    expect(augnitoRecipe.nextPrevious, "last");
    expect(augnitoRecipe.selectFor, "gotostart");
  });

  test("delete the previous 2 words", () {
    var augnitoRecipe =
        ActionRecipe(receivedText: "delete the previous 2 words");
    AugnitoCommandBuilder.build(augnitoRecipe);
    expect(augnitoRecipe.name, AugnitoCommands.selectWord);
    expect(augnitoRecipe.isCommand, true);
    expect(augnitoRecipe.chooseNumber, 2);
    expect(augnitoRecipe.receivedText, "");
    expect(augnitoRecipe.nextPrevious, "previous");
    expect(augnitoRecipe.selectFor, "delete");
  });

  test("italicize previous 9 characters", () {
    var augnitoRecipe =
        ActionRecipe(receivedText: "italicize previous 9 characters");
    AugnitoCommandBuilder.build(augnitoRecipe);
    expect(augnitoRecipe.name, AugnitoCommands.selectChar);
    expect(augnitoRecipe.isCommand, true);
    expect(augnitoRecipe.chooseNumber, 9);
    expect(augnitoRecipe.receivedText, "");
    expect(augnitoRecipe.nextPrevious, "previous");
    expect(augnitoRecipe.selectFor, "italicize");
  });

  test("copy text next 3 paras", () {
    var augnitoRecipe = ActionRecipe(receivedText: "copy text next 3 paras");
    AugnitoCommandBuilder.build(augnitoRecipe);
    expect(augnitoRecipe.name, AugnitoCommands.selectParagraph);
    expect(augnitoRecipe.isCommand, true);
    expect(augnitoRecipe.chooseNumber, 3);
    expect(augnitoRecipe.receivedText, "");
    expect(augnitoRecipe.nextPrevious, "next");
    expect(augnitoRecipe.selectFor, "copy");
  });

  test("replace cat with dog", () {
    var augnitoRecipe = ActionRecipe(receivedText: "replace cat with dog");
    AugnitoCommandBuilder.build(augnitoRecipe);
    expect(augnitoRecipe.name, AugnitoCommands.replace);
    expect(augnitoRecipe.isCommand, true);
    expect(augnitoRecipe.searchText, "cat");
    expect(augnitoRecipe.selectFor, "dog");
  });

  test("select developer", () {
    var augnitoRecipe = ActionRecipe(receivedText: "select developer");
    AugnitoCommandBuilder.build(augnitoRecipe);
    expect(augnitoRecipe.name, AugnitoCommands.select);
    expect(augnitoRecipe.isCommand, true);
    expect(augnitoRecipe.searchText, "developer");
    expect(augnitoRecipe.selectFor, "");
  });

  test("bold important", () {
    var augnitoRecipe = ActionRecipe(receivedText: "bold important");
    AugnitoCommandBuilder.build(augnitoRecipe);
    expect(augnitoRecipe.name, AugnitoCommands.select);
    expect(augnitoRecipe.isCommand, true);
    expect(augnitoRecipe.searchText, "important");
    expect(augnitoRecipe.selectFor, "bold");
  });

  test("copy text some sentence", () {
    var augnitoRecipe = ActionRecipe(receivedText: "copy text some sentence");
    AugnitoCommandBuilder.build(augnitoRecipe);
    expect(augnitoRecipe.name, AugnitoCommands.select);
    expect(augnitoRecipe.isCommand, true);
    expect(augnitoRecipe.searchText, "somesentence");
    expect(augnitoRecipe.selectFor, "copy");
  });

  test("number 21", () {
    var augnitoRecipe = ActionRecipe(receivedText: "number 21");
    AugnitoCommandBuilder.build(augnitoRecipe);
    expect(augnitoRecipe.name, AugnitoCommands.number);
    expect(augnitoRecipe.isCommand, false);
    expect(augnitoRecipe.chooseNumber, 21);
  });

  test("12", () {
    var augnitoRecipe = ActionRecipe(receivedText: "12");
    AugnitoCommandBuilder.build(augnitoRecipe);
    expect(augnitoRecipe.name, AugnitoCommands.number);
    expect(augnitoRecipe.isCommand, false);
    expect(augnitoRecipe.chooseNumber, 12);
  });

  test("select active paragraph", () {
    var augnitoRecipe = ActionRecipe(receivedText: """select active
    
    """);
    AugnitoCommandBuilder.build(augnitoRecipe);
    expect(augnitoRecipe.name, AugnitoCommands.selectActiveParagraph);
    expect(augnitoRecipe.isCommand, true);
  });

  test("select the active character", () {
    var augnitoRecipe =
        ActionRecipe(receivedText: "select the active character");
    AugnitoCommandBuilder.build(augnitoRecipe);
    expect(augnitoRecipe.name, AugnitoCommands.selectActiveChar);
    expect(augnitoRecipe.isCommand, true);
  });
}
